/****************************************************************************/
/*
 * MODULE              JN-AN-1162 JenNet-IP Smart Home
 *
 * DESCRIPTION         IlluminanceScene MIB Interface
 */
/****************************************************************************/
/*
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5164].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2014. All rights reserved
 */
/****************************************************************************/
#ifndef  MIBILLUMINANCESCENE_H_INCLUDED
#define  MIBILLUMINANCESCENE_H_INCLUDED

#if defined __cplusplus
extern "C" {
#endif

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/
/* SDK includes */
#include <jendefs.h>
/* JenOS includes */
#include <os.h>
#include <pdm.h>
/* Application common includes */
#include "Table.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
#define MIB_ILLUMINANCE_SCENE_SCENES					16

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
/* Scene data structure */
#pragma GCC diagnostic ignored "-Wpacked"
#pragma GCC diagnostic ignored "-Wattributes"
typedef struct
{
	uint16 u16Id;
	uint16 u16LuxTarget;
	uint16 u16LuxBand;
} PACK tsIlluminanceScene;
#pragma GCC diagnostic pop

/* Illuminance configuration permament data */
typedef struct
{
	/* Config MIB variable data */
   	tsIlluminanceScene asSceneTable[MIB_ILLUMINANCE_SCENE_SCENES];

}  tsMibIlluminanceScenePerm;

/* Illuminance configuration temporary data */
typedef struct
{
	/* Config MIB variable data */
	uint16 			   u16AddSceneId;
	uint16 			   u16DelSceneId;
	tsIlluminanceScene	 sSetScene;

} tsMibIlluminanceSceneTemp;

/* Illuminance configuration mib */
typedef struct
{
	/* MIB handle */
	thJIP_Mib 	hMib;

	/* Table handle */
   	tsTable sSceneTable;

	/* PDM record descriptor */
	PDM_tsRecordDescriptor  sDesc;

	/* Data pointers */
	tsMibIlluminanceScenePerm 	sPerm;
	tsMibIlluminanceSceneTemp 	sTemp;

	/* Other MIB pointers */
	void *pvMibIlluminanceControl;

} tsMibIlluminanceScene;

/****************************************************************************/
/***        Public Function Prototypes                                     ***/
/****************************************************************************/
PUBLIC void 		MibIlluminanceScene_vInit(thJIP_Mib         hMibIlluminanceSceneInit,
						   				tsMibIlluminanceScene   *psMibIlluminanceSceneInit,
						   				void 			 		*pvMibIlluminanceControlInit);
PUBLIC void 		MibIlluminanceScene_vRegister(void);
PUBLIC uint8 		MibIlluminanceScene_u8FindSceneId(uint16 u16SceneId);
PUBLIC bool_t 		MibIlluminanceScene_bSetScene(uint16 u16Id, uint16 u16LuxTarget, uint16 u16LuxBand);
PUBLIC teJIP_Status	MibIlluminanceScene_eSetAddSceneId(uint16 u16Val, void *pvCbData);
PUBLIC teJIP_Status MibIlluminanceScene_eSetDelSceneId(uint16 u16Val, void *pvCbData);
PUBLIC teJIP_Status MibIlluminanceScene_eSetSetScene(const uint8 *pu8Val, uint8 u8Len, void *pvCbData);
PUBLIC void 		MibIlluminanceScene_vGetSetScene(thJIP_Packet hPacket, void *pvCbData);

#if defined __cplusplus
}
#endif

#endif  /* MIBILLUMINANCESCENE_H_INCLUDED */
/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
